/*
JMP Clinical Analytical Process

Name: ClusterSubjectsAcrossSite
Authors: Richard C. Zink, SAS Institute Inc.

Description: 

Performs Cluster subjects across study sites within race, gender and country (if requested).
	
*/

/*
-----------------------------------------------------------
Copyright (C) 2013 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;
*options nomprint nomlogic nosymbolgen;

%global StudyName InPath AInPath SubjectFilter TrtVar TrtUse MacroPath OutName ClusterSubAcrossScript ClusterSubAcross_OutData OutPath 
        ServerOutPath PopulationSet MissMax FindDistFl InterDistFl EventDistFl WhereStmt WhereForReport DelMissFl HierClustMethod ClustSexFl ClustRaceFl ClustCountryFl AgeSexFl;

*ProcessBody;

%macro ClusterSubjectsAcrossSite;

*initialization and argument checking;

%let ProcessName = ClusterSubjectsAcrossSite; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%if &exiterror %then %goto exit;
%let WhereForReport=%nrbquote(&WhereStmt);
%if &AgeSexFl = No and &EventDistFl = No and &InterDistFl = No and &FindDistFl = No %then %do;
	%put ERROR: At least one set of variables must be selected for analysis.;
	%let exiterror = 1;
	%goto exit;
%end;

%let trCountry = %nrbquote(%LocaleGet(Country_Text));
%let trNotProvided = %nrbquote(%LocaleGet(NotProvided_Text));
%let trSex = %nrbquote(%LocaleGet(Sex_Text));
%let trRace = %nrbquote(%LocaleGet(Race_Text));
%let trAllSubjects = %nrbquote(%LocaleGet(AllSubjects_Text));
%let trCovariateGroupID = %nrbquote(%LocaleGet(CovariateGroupID_Text));
%let tryear = %nrbquote(%LocaleGet(year_Text));
%let tryears = %nrbquote(%LocaleGet(years_Text));

%if %sysfunc(findw(%upcase(&ADSLVARS), AGE, ' ')) %then %let ageflag = age; 
%else %let ageflag = ;

%if %sysfunc(findw(%upcase(&ADSLVARS), SEX, ' ')) %then %let sexflag = sex;
%else %let sexflag = ;

%if %sysfunc(findw(%upcase(&ADSLVARS), RACE, ' ')) %then %let raceflag = race;
%else %let raceflag = ;

%if %upcase(&ClustSexFl) = NO and %upcase(&ClustRaceFl) = NO and %upcase(&ClustCountryFl) = NO %then %let OverAll = YES;
%else %let OverAll = NO;

%let yeardiff =;
%let yeardifffl =;
%if %sysfunc(findw(&domains, DM, ' ')) %then %do;
	%if %sysfunc(findw(%upcase(&DMVARS), RFSTDTC, ' ')) %then %let flagdate = RFSTDTC;
	%else %if %sysfunc(findw(%upcase(&DMVARS), DMDTC, ' ')) %then %let flagdate = DMDTC;
	%else %let flagdate = ;

   %if %length(&flagdate) %then %do;
   	data dm2(keep = &flagdate);
   	   set &in_dm;
   	run;

   	proc sort nodupkey data = dm2 out = dm2;
   	   by &flagdate;
   	run;
   	
   	data dm2(keep = first last);
   	   set dm2(where = (not missing(&flagdate))) end = eof;
   	   retain first last;
   	   if _n_ = 1 then first = &flagdate;
   	   if eof then last = &flagdate;
   	   if eof;
   	run;
   	
   	data dm2;
   	   set dm2;
   	   yeardiff = 1 + substr(last,1,4) - substr(first,1,4);
   	   call symput("yeardiff",trim(left(yeardiff)));
   	   if yeardiff = 1 then call symput("yeardifffl","&tryear");
   	   else call symput("yeardifffl","&tryears");
      run;

      proc datasets nolist;
         delete dm2;
      quit;   
   %end;
%end;

%let _tid_ = %sysfunc(open(&in_adsl));
%let vn = %sysfunc(varnum(&_tid_,USUBJID));
%let USubLen = %sysfunc(varlength(&_tid_,&vn));
%let _tid_ = %sysfunc(close(&_tid_));

%if ^%length(&findingsdomains) and %upcase(&FindDistFl) = YES %then %do;
	%put ERROR: No Findings domains are available for the current study.;
	%let exiterror = 1;
	%goto exit;
%end;

%if ^%length(&interventionsdomains) and %upcase(&InterDistFl) = YES %then %do;
	%put ERROR: No Interventions domains are available for the current study.;
	%let exiterror = 1;
	%goto exit;
%end;

%if ^%length(&eventsdomains) and %upcase(&EventDistFl) = YES %then %do;
	%put ERROR: No Events domains are available for the current study.;
	%let exiterror = 1;
	%goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), ARM, ' ')) %then %do;
	%put ERROR: ARM is not available in either DM, ADDM or ADSL.;
	%let exiterror = 1;
	%goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), SITEID, ' ')) %then %do;
	%put ERROR: SITEID is not available in either DM, ADDM or ADSL.;
	%let exiterror = 1;
	%goto exit;
%end;

%let _tid_ = %sysfunc(open(&in_adsl));
%let usubjvn = %sysfunc(varnum(&_tid_,USUBJID));
%let SubLab = %sysfunc(varlabel(&_tid_,&usubjvn));
%let sitevn = %sysfunc(varnum(&_tid_,SITEID));
%let SiteLab = %sysfunc(varlabel(&_tid_,&sitevn));
%let _tid_ = %sysfunc(close(&_tid_));  
    	
%if %index(%upcase(&in_adsl), ADSL) and %sysfunc(findw(%upcase(&ADSLVARS), COUNTRY, ' ')) = 0 and %sysfunc(findw(%upcase(&DMVARS), COUNTRY, ' ')) > 0 %then %do;
	proc sort data = &in_adsl out = subject(keep = usubjid siteid arm &ageflag &sexflag &raceflag);
		by usubjid;
	run;

	proc sort data = &in_dm out = dm;
		by usubjid;
	run;

	data subject;
		merge subject(in = ina) dm(keep = usubjid country);
		by usubjid;
		if ina;
	run;
%end;
%else %if %sysfunc(findw(%upcase(&ADSLVARS), COUNTRY, ' ')) %then %do;
	proc sort data = &in_adsl out = subject(keep = usubjid siteid arm country &ageflag &sexflag &raceflag);
		by usubjid;
	run;
%end;
%else %do;
	proc sort data = &in_adsl out = subject(keep = usubjid siteid arm &ageflag &sexflag &raceflag);
		by usubjid;
	run;
	
	data subject;
	   length Country $ 12;
	   set subject;
	   country = "&trNotProvided";
	   label country = "&trCountry";
	run;
%end;

data subject;
   set subject;
   int = 1;
run;

proc means sum nway noprint data = subject;
   class 
		%if %upcase(&ClustSexFl) = YES and %sysfunc(findw(%upcase(&ADSLVARS), SEX, ' ')) %then %do; sex %end;
		%if %upcase(&ClustRaceFl) = YES and %sysfunc(findw(%upcase(&ADSLVARS), RACE, ' ')) %then %do; race %end;
		%if %upcase(&ClustCountryFl) = YES %then %do; country %end;
   ;
   var int;
   output out = covgrp(drop = _TYPE_ _FREQ_) sum = count;
run;

proc sort data = covgrp;
   by descending count;
run;

data covgrp;
   set covgrp;
   covid = catx(", ", 
      %if &Overall = NO %then %do;
			%if %upcase(&ClustSexFl) = YES and %sysfunc(findw(%upcase(&ADSLVARS), SEX, ' ')) %then %do; catx(" ","&trSex =",sex), %end;
			%if %upcase(&ClustRaceFl) = YES and %sysfunc(findw(%upcase(&ADSLVARS), RACE, ' ')) %then %do; catx(" ","&trRace =",race), %end;       
			%if %upcase(&ClustCountryFl) = YES %then %do; catx(" ","&trCountry =",country), %end;       
      	" "
      %end;
      %else %do;
      	"&trAllSubjects"
      %end;
   );
   label covid = "&trCovariateGroupID";
run;

%if &Overall = NO %then %do;
	proc sort data = subject;
		by 
			%if %upcase(&ClustSexFl) = YES and %sysfunc(findw(%upcase(&ADSLVARS), SEX, ' ')) %then %do; sex %end;
			%if %upcase(&ClustRaceFl) = YES and %sysfunc(findw(%upcase(&ADSLVARS), RACE, ' ')) %then %do; race %end;
			%if %upcase(&ClustCountryFl) = YES %then %do; country %end;
		;
	run;

	proc sort data = covgrp;
		by 
			%if %upcase(&ClustSexFl) = YES and %sysfunc(findw(%upcase(&ADSLVARS), SEX, ' ')) %then %do; sex %end;
			%if %upcase(&ClustRaceFl) = YES and %sysfunc(findw(%upcase(&ADSLVARS), RACE, ' ')) %then %do; race %end;
			%if %upcase(&ClustCountryFl) = YES %then %do; country %end;
		;
	run;

	data subject;
		merge subject covgrp(drop = count);
		by
			%if %upcase(&ClustSexFl) = YES and %sysfunc(findw(%upcase(&ADSLVARS), SEX, ' ')) %then %do; sex %end;
			%if %upcase(&ClustRaceFl) = YES and %sysfunc(findw(%upcase(&ADSLVARS), RACE, ' ')) %then %do; race %end;
			%if %upcase(&ClustCountryFl) = YES %then %do; country %end;   
		;
	run;

	proc sort data = subject out = subject(drop = int);
		by usubjid;
	run;
%end;
%else %do;
   data subject(drop = int);
      set subject;
      if _n_ = 1 then set covgrp(drop = count);
   run;
%end;

%let trUnscheduled = %nrbquote(%LocaleGet(Unscheduled_Text));

%let htwtobs = 0;
%let heightflag = 0;
%let weightflag = 0;
%if %sysfunc(findw(&domains, VS, ' ')) %then %do;
   %let temp = VS;
   %if %sysfunc(indexw(&&&temp.VARS, &temp.STRESN, ' ')) %then %do;
		%if %sysfunc(indexw(&&&temp.VARS, &temp.DTC, ' ')) %then %let dtc = &temp.DTC;
		%else %let dtc = ;

		%if %sysfunc(indexw(&&&temp.VARS, &temp.test, ' ')) %then %let testlab = &temp.test;
		%else %let testlab = ;

		proc sort data = &&in_&temp(where = (not missing (&temp.STRESN))) out = &temp.2(keep = usubjid &temp.testcd &testlab &temp.STRESN &dtc);
			by usubjid &temp.testcd &dtc;
		run;

		*** Delete unscheduled records ***;
		%if %sysfunc(indexw(&&&temp.VARS, VISIT, ' ')) %then %do;
			data &temp.2(drop = visit);
				set &temp.2;
				if index(upcase(visit), "UNSCHEDULED") or kindex(kupcase(visit), "%kupcase(&trUnscheduled)") then delete;
			run;
		%end;

		*** if possible, limit measurements to those prior to dosing ***;
		%if %length(&dtc) %then %do;
			%TREATMENTEMERGENT(dataset = &temp.2, domain = &temp, date = &temp.DTC);

			data &temp.2(drop = &dtc _TRTEMRG);
				set &temp.2;
				where _TRTEMRG = 0;
			run;
		%end;

		ods exclude all;
		ods noresults;
		proc means noprint nway missing data = &temp.2;
			class usubjid &temp.testcd &testlab;
			var &temp.STRESN;
			output out = &temp.2(drop = _type_ _freq_) mean = &temp.STRESN;
			where upcase(&temp.testcd) in ("HEIGHT","WEIGHT");
		run;
		ods select all;
		ods results;      

		proc transpose data = &temp.2 out = htwt(drop = _name_ _label_);
			by usubjid;
			var &temp.STRESN;
			id &temp.testcd;
			%if %length(&testlab) %then %do;
				idlabel &testlab;
			%end;
		run;		
		%NObsVars(htwt);
		%let htwtobs = &nobs;

      proc datasets nolist;
         delete &temp.2;
      quit;		
   %end;
%end;

%if &htwtobs %then %do;
   data subject;
      merge subject(in = ina) htwt;
      by usubjid;
      if ina;
   run;

	%let _tid_ = %sysfunc(open(subject));
	%let HeightFlag = %sysfunc(varnum(&_tid_,HEIGHT));
	%let WeightFlag = %sysfunc(varnum(&_tid_,WEIGHT));
	%let _tid_ = %sysfunc(close(&_tid_));  
%end;

proc sql noprint;
   create table subpairs as
      select aa.usubjid, bb.usubjid2, aa.covid, aa.siteid, bb.siteid2 %if %length(&ageflag) %then %do; , aa.age, bb.age2 %end; %if &heightflag %then %do; , height , height2 %end; %if &weightflag %then %do; , weight ,weight2 %end;
      from subject as aa, subject(rename = (usubjid = USUBJID2 covid = covid2 siteid = siteid2 %if %length(&ageflag) %then %do; age = age2 %end; %if &heightflag %then %do; height = height2 %end; %if &weightflag %then %do; weight = weight2 %end;)) as bb 	
      where aa.usubjid < bb.usubjid2 and covid = covid2
      order by aa.usubjid, bb.usubjid2;
quit;

%let trWithin = %nrbquote(%LocaleGet(Within_Text));
%let trBetween = %nrbquote(%LocaleGet(Between_Text));
%let trSiteFlag = %nrbquote(%LocaleGet(SiteFlag_Text));
%let trAgeDifference = %nrbquote(%LocaleGet(AgeDifference_Text));
%let trHeightDifference = %nrbquote(%LocaleGet(HeightDifference_Text));
%let trWeightDifference = %nrbquote(%LocaleGet(WeightDifference_Text));

data subpairs;
   retain USUBJID USUBJID2 covid siteflag %if %length(&ageflag) %then %do; agediff %end; %if &heightflag %then %do; heightdiff %end; %if &weightflag %then %do; weightdiff %end;;
   length siteflag $ 20;
   set subpairs;
   if siteid = siteid2 then siteflag = "&trWithin";
   else siteflag = "&trBetween";
   label siteflag = "&trSiteFlag";
   drop siteid siteid2;
   %if %length(&ageflag) %then %do;
      agediff = abs(age-age2);
      drop age age2;
      label agediff = "&trAgeDifference";
   %end;   
   %if &heightflag %then %do; 
      heightdiff = abs(height-height2);
      drop height height2;
      label heightdiff = "&trHeightDifference";   
   %end; 
   %if &weightflag %then %do; 
      weightdiff = abs(weight-weight2);
      drop weight weight2;
      label weightdiff = "&trWeightDifference";      
   %end;
run;

data subject;
   set subject;
	%if &heightflag %then %do; 
		drop height;
	%end; 
	%if &weightflag %then %do; 
		drop weight;
	%end;   
run;

%let availabledomains = ;

*** create by lists for each domain ***;
%if &FindDistFl = Yes %then %do;
	%let i = 1;
	%do %while(%length(%scan(&findingsdomains,&i,' ')));
		%let temp = %scan(&findingsdomains,&i,' ');

		%if %sysfunc(indexw(&&&temp.VARS, VISITNUM, ' ')) and %sysfunc(indexw(&&&temp.VARS, &temp.STRESN, ' ')) %then %do;
			%let availabledomains = &availabledomains &temp;
			%if %sysfunc(indexw(&&&temp.VARS, &temp.TPTNUM, ' ')) %then %let tpt = &temp.tptnum;
			%else %let tpt =;

			%if %sysfunc(indexw(&&&temp.VARS, VISIT, ' ')) %then %let visit = visit;
			%else %let visit = ;
			
			%if %sysfunc(indexw(&&&temp.VARS, &temp.DTC, ' ')) %then %let dtc = &temp.DTC;
			%else %let dtc = ;
			
			proc sort data = &&in_&temp(where = (not missing (&temp.STRESN))) out = &temp(keep = usubjid &temp.testcd &temp.STRESN &dtc &tpt VISITNUM &visit);
				by usubjid &temp.testcd visitnum &tpt;
			run;

			*** Delete unscheduled records ***;
			%if %sysfunc(indexw(&&&temp.VARS, VISIT, ' ')) %then %do;
				data &temp(drop = visit);
					set &temp;
					if index(upcase(visit), "UNSCHEDULED") or kindex(kupcase(visit), "%kupcase(&trUnscheduled)") then delete;
				run;
			%end;

         *** if possible, limit measurements to those prior to dosing ***;
         %if %length(&dtc) %then %do;
	         %TREATMENTEMERGENT(dataset = &temp, domain = &temp, date = &temp.DTC);
            
				data &temp(drop = &dtc _TRTEMRG);
					set &temp;
					where _TRTEMRG = 0;
				run;
			%end;
			
			ods exclude all;
			ods noresults;
			proc means noprint nway missing data = &temp;
				class usubjid &temp.testcd visitnum &tpt;
				var &temp.STRESN;
				output out = &temp(drop = _type_ _freq_) mean = &temp.STRESN;
			run;

			ods select all;
			ods results;

			data &temp;
				set &temp;
				%if %length(&tpt) %then %do;
				   if missing(&tpt) then level = cats(&temp.testcd,"_","V",visitnum);         
				   else level = cats(&temp.testcd,"_","V",visitnum,"_","T",&tpt);         
				%end;
				%else %do;
					level = cats(&temp.testcd,"_","V",visitnum);
				%end;
			run;

			proc sort data = &temp out = &temp;
				by usubjid level;
			run;
			
			proc transpose data = &temp out = &temp(drop = _name_ _label_);
				by usubjid;
				var &temp.STRESN;
				id level;
				idlabel level;
			run;              
		%end;
		%let i = %eval(&i+1);
	%end;
%end;

%macro INTEVENT(list = );
	%let i = 1;
	%do %while(%length(%scan(&list,&i,' ')));
		%let temp = %scan(&list,&i,' ');

		%if %sysfunc(indexw(&&&temp.VARS, &temp.DECOD, ' '))  %then %do;
			%let availabledomains = &availabledomains &temp;

			proc sort data = &&in_&temp(where = (not missing (&temp.DECOD))) out = &temp;
				by usubjid &temp.DECOD;
			run;

         %if %sysfunc(indexw(&&&temp.VARS, &temp.STDTC, ' ')) and &temp ne MH %then %let dtc = &temp.STDTC;
         %else %let dtc = ;
         
			*** Delete prespecified not occuring records ***;
			data &temp(keep = usubjid &temp.DECOD count &dtc);
				set &temp;
				count = 1;
				%PRESPECIFY(dom = &temp);
			run;

         *** if possible, limit measurements to those prior to dosing ***;
         %if %length(&dtc) %then %do;
	         %TREATMENTEMERGENT(dataset = &temp, domain = &temp, date = &temp.STDTC);
         
				data &temp(drop = &temp.STDTC _TRTEMRG);
					set &temp;
					where _TRTEMRG = 0;
				run;
			%end;
			
         proc freq noprint data = &temp;
            tables &temp.DECOD / out = outname(drop = count percent);
         run;
                  
         data outname;
            length &temp.short $ 8;
            set outname;
            &temp.short = cats("&temp",_n_);   
         run;

         proc sort data = &temp;
            by usubjid &temp.DECOD;
			run;

			ods exclude all;
			ods noresults;
			proc means noprint nway data = &temp;
				class usubjid &temp.DECOD;
				var count;
				output out = &temp(drop = _type_ _freq_) n = count;
			run;
			ods select all;
			ods results;

			proc sql noprint;      
				create table events as
					select distinct &temp.DECOD
					from &temp
					order by &temp.DECOD;		
			quit;

			proc sql;
				create table events1 as
				select aa.usubjid, bb.&temp.DECOD
				from subject as aa, events as bb
				order by aa.usubjid, bb.&temp.DECOD;
			quit;

			data &temp;
			  merge &temp(in = ina) events1(in = inb);
			  by usubjid &temp.DECOD;
			  if inb and not ina then count = 0;		  
			run;

         proc sort data = &temp;
            by &temp.decod;
         run;
           
         data &temp;
            merge &temp outname;
            by &temp.decod;
         run;

			proc datasets nolist;
				delete events events1 outname;
			quit;
			
         proc sort data = &temp;
            by usubjid &temp.decod;
         run;			
           
			proc transpose data = &temp out = &temp(drop = _name_ _label_);
				by usubjid;
				var count;
				id &temp.short; 
				idlabel &temp.DECOD;
			run;   
		%end;

		%let i = %eval(&i+1);
	%end;
%mend INTEVENT;

%if &EventDistFl = Yes %then %do;
	%INTEVENT(list = &eventsdomains);
%end;

%if &InterDistFl = Yes %then %do;
	%INTEVENT(list = &interventionsdomains); 
%end;

%let trMaleGender = %nrbquote(%LocaleGet(MaleGender_Text));
%let trM = %nrbquote(%LocaleGet(M_Text));
%let trF = %nrbquote(%LocaleGet(F_Text));

data all;
   merge subject(in = ina) &availabledomains;
   by usubjid;
   if ina;
   %if %length(&sexflag) %then %do;
      if sex = "M" or sex = "&trM" then MALE = 1;
      else if sex = "F" or sex = "&trF" then MALE = 0;
      drop sex;
      label male = "&trMaleGender";
   %end;
	%if &AgeSexFl = No %then %do;
		%if %length(&ageflag) %then %do;
      	drop age;
		%end;
		%if %length(&sexflag) %then %do;
			drop male;
		%end;
	%end;
run;
%NObsVars(all);
%let TotalSubs = &nobs;

proc means nmiss noprint data = all;
	output out = nmiss(drop = _type_ _freq_) nmiss = / autoname;
run;

proc transpose data = nmiss out = nmiss;
run;

%let trVariable = %nrbquote(%LocaleGet(Variable_Text));
%let trLabel = %nrbquote(%LocaleGet(Label_Text));
%let trNOSMV = %nrbquote(%LocaleGet(NumberofSubjectsMissingValues_Text));
%let trPOSMV = %nrbquote(%LocaleGet(PercentageofSubjectsMissingValues_Text));
%let trMissFlag = %nrbquote(%LocaleGet(MissingFlag_Text));

data nmiss;
   set nmiss(rename = (col1 = nummiss)) end = eof;
   _name_ = substr(_name_, 1, length(_name_) - 6);
   pctmiss = round(nummiss / &TotalSubs * 100, 0.01);
   %if %upcase(&DelMissFl) = YES %then %do; 
   	if pctmiss > 0 then missflag = 2;
   %end;
   %else %do;
   	if pctmiss >= &MissMax then missflag = 2;
   %end;
   else if pctmiss ne 0 then missflag = 1;
   else if pctmiss = 0 then missflag = 0;
   if eof then call symput("TotalVars", trim(left(_n_)));
   label _name_ = "&trVariable" _label_ = "&trLabel" nummiss = "&trNOSMV" pctmiss = "&trPOSMV" missflag = "&trMissFlag";
run;

%let DegFree = 0;
*** degree of freeom is number of variables contributing to maha scores;
data _null_;
   set nmiss(where = (missflag in (0,1))) end = eof;
   if eof then call symput("DegFree", trim(left(_n_)));
run;

%let RemovedVars = ;
data _null_;
   set nmiss(where = (missflag = 2)) end = eof;
   if eof then call symput("RemovedVars", trim(left(_n_)));
run;

%if ^%length(&RemovedVars) %then %let RemovedVars = 0;
%if ^%eval(&TotalVars - &RemovedVars) %then %do;
	%put ERROR: No variables remain for analysis after deleting variables with at least &MissMax percent missing data.;
	%let exiterror = 1;
	%goto exit;
%end;
%let OtherVars = %eval(&TotalVars - &RemovedVars - &DegFree);

%let MissMaxVars = ;
%let MissMaxLabels = ;
%let MissLabels = ;
proc sql noprint;
	select _name_ 
		into :MissMaxVars separated by ' '
		from nmiss
		where missflag = 2
		order by _name_;
quit;

data all;
   set all(drop = &MissMaxVars);
   subject = cats("S",_n_);
run;

%let includeds =;

%let ClusterSubAcross_OutData_sum = &ClusterSubAcross_OutData;	   
%PrepOutDS(ClusterSubAcross_OutData_sum,OutDataName_sum, clustsubs_summary);
%let includeds = &includeds &OutDataName_sum..sas7bdat,;
	
data &ClusterSubAcross_OutData_sum;
   set all(drop = subject) end = eof;
run;

%let SiteList = ;
proc sql noprint;
	select distinct covid 
		into :SiteList separated by '~'
		from covgrp(where = (count >= 2))
		order by covid;
quit;

proc means var nway noprint data = all;
	class covid;
	output out = novar(drop = _type_ _freq_) var = / autoname;
run;

proc transpose data = novar out = novar;
   by covid;
run;

proc sql noprint;
   create table SiteVarCountList as
	select distinct covid, count(covid) as countvars 
		from novar
		group by covid
		order by covid;
quit;

data novar;
   set novar(rename = (col1 = var) where = (var in (.,0))) end = eof;
   _name_ = substr(_name_, 1, length(_name_) - 4);
run;

proc sql noprint;
   create table SiteNoVarCountList as
	select distinct covid, count(covid) as countnovar
		from novar
		group by covid
		order by covid;
quit;

data SiteVarCountList;
    merge SiteVarCountList SiteNoVarCountList;
    by covid;
run;

proc datasets nolist;
   delete SiteNoVarCountList;
quit;

%let SitesNoVariability =;
%let firstflag = 0;

%let trDistance = %nrbquote(%LocaleGet(Distance_Text));

%let i = 1;
%do %while(%length(%scan(%nrbquote(&SiteList),&i,'~')));
	%let tempsite = %scan(%nrbquote(&SiteList),&i,'~');	
	%let tempsiten = &i;	
	%let SiteConstantVars = ;	
	proc sql noprint;
		select _name_ 
			into :SiteConstantVars separated by ' '
			from novar
			where covid = "&tempsite"
			order by _name_;
	quit;

   %let varok = 1;
   data _null_;
      set SiteVarCountList(where = (covid = "&tempsite"));
      if countvars = countnovar then do;
      	call symput("varok",trim(left(0)));
      end;
   run;
   
   %if ^&varok %then %let SitesNoVariability = &SitesNoVariability~&tempsite;
    
   %if &varok %then %do;   
		proc distance data = all(drop = &SiteConstantVars) shape = square method = euclid out = distance;
			var interval(_numeric_ / std = std);
			copy usubjid covid siteid arm country;
			id subject;
			where covid = "&tempsite";
		run;

		proc sql noprint;
			select usubjid 
				into :SiteClusterVars separated by ' '
				from distance
				order by usubjid;
		quit;

		%let SiteClusterVars&tempsiten = ;

		%let j = 1;
		%do %while(%length(%scan(&SiteClusterVars,&j,' ')));
			%let temp = %scan(%upcase(&SiteClusterVars),&j," ");      
			%let SiteClusterVars&tempsiten = &&SiteClusterVars&tempsiten %bquote(:Name(""&temp"")),;
			%let j = %eval(&j+1);
		%end;

		proc transpose data = distance out = distance;
			by usubjid subject covid siteid arm country;
		run;

		proc sql noprint;
			create table siteids as
				select distinct subnum, usubjid2
				from all(rename = (subject = subnum usubjid = USUBJID2))
				order by subnum;		
			create table distance2 as
				select * 
				from distance as aa left join siteids as bb
				on aa._name_ = bb.subnum
				order by usubjid, usubjid2;
		quit;

		proc datasets nolist;
			delete siteids distance;
			change distance2 = distance;
		quit;

		proc sql noprint;
			create table distancepairs(drop = subnum siteid subject _name_ rename = (col1 = Distance)) as
				select *, min(col1) as mindist
				from subpairs(where = (covid = "&tempsite")) natural left join distance
				order by usubjid, usubjid2;		
		quit;

		data mindisttemp(drop = mindist);
			set distancepairs(where = (distance = mindist));
			if _n_ = 1;
		run;

		%if ^&firstflag %then %do;
			data mindist;
				set mindisttemp;
			run;
		%end;
		%else %do;   
			proc append force base = mindist data = mindisttemp;
			run;
		%end;

		data distancepairs;
			set distancepairs(drop = mindist);
			label distance = "&trDistance" usubjid = "&SubLab" usubjid2 = "&SubLab 2" covid = "&trCovariateGroupID";
		run;

		%if ^&firstflag %then %do;
			data alldistancepairs;
				set distancepairs;
			run;
		%end;
		%else %do;   
			proc append force base = alldistancepairs data = distancepairs;
			run;
		%end;

		%if ^&firstflag %then %let firstflag = 1;

		%if &i = 1 %then %do;   
			%let SUBJECTLabel2 = USUBJID2;
			%CheckLabel(distancepairs,SUBJECTLabel2);
		%end;

		proc transpose data = distance out = distance(drop = _name_);
			var col1;
			by usubjid siteid arm country;
			id _name_;
			idlabel usubjid2;
		run;

		%let ClusterSubAcross_OutData_&tempsiten = &ClusterSubAcross_OutData;	   
		%let ClusterSubAcross_OutDataP_&tempsiten = &ClusterSubAcross_OutData;	   

		%PrepOutDS(ClusterSubAcross_OutData_&tempsiten,OutDataName_&tempsiten, clustsubs_site, _&tempsiten);
		%PrepOutDS(ClusterSubAcross_OutDataP_&tempsiten,OutDataNameP_&tempsiten, clustsubspairs_site, _&tempsiten);

		%let includeds = &includeds &&&OutDataName_&tempsiten...sas7bdat, &&&OutDataNameP_&tempsiten...sas7bdat,;

		data &&&ClusterSubAcross_OutData_&tempsiten;
			set distance;
		run;

		data &&&ClusterSubAcross_OutDataP_&tempsiten;
			set distancepairs;
		run;

		proc datasets nolist;
			delete distance distancepairs mindisttemp;	 
		quit;
	%end;	
   %let i = %eval(&i+1);
%end; 

%if %sysfunc(exist(mindist)) %then %do;
	proc sort data = mindist;
		by distance;
	run;

	%let ClusterSubAcross_OutData_min = &ClusterSubAcross_OutData;	   
	%PrepOutDS(ClusterSubAcross_OutData_min,OutDataName_min, clustsubs_across_mindist);
	%let includeds = &includeds &OutDataName_min..sas7bdat,;

	data &ClusterSubAcross_OutData_min;
		set mindist end = eof;
		if _n_ = 1 then call symput("MinDistSite", trim(covid));
		label distance = "&trDistance" usubjid = "&SubLab" usubjid2 = "&SubLab 2" covid = "&trCovariateGroupID";
	run;
%end;

%if %sysfunc(exist(alldistancepairs)) %then %do;
	%let ClusterSubAcross_OutData_all = &ClusterSubAcross_OutData;	   
	%PrepOutDS(ClusterSubAcross_OutData_all,OutDataName_all, clustsubs_across_alldist);
	%let includeds = &includeds &OutDataName_all..sas7bdat,;

	data &ClusterSubAcross_OutData_all;
		set alldistancepairs end = eof;
	run;

	proc datasets nolist;
		delete alldistancepairs;
	quit;
%end;

%let ARMLabel = ARM;
%CheckLabel(all,ARMLabel);
%let SUBJECTLabel = USUBJID;
%CheckLabel(all,SUBJECTLabel);
%let COUNTRYLabel = COUNTRY;
%CheckLabel(all,COUNTRYLabel);
%let COVIDlabel = COVID;
%CheckLabel(all,COVIDlabel);
%let SITEIhlabel = SITEID;
%CheckLabel(all,SITEIhlabel);

proc datasets nolist;
   delete all;
quit;		

* create JSL file;
%PrepOutFile(ClusterSubAcrossScript,ClusterSubAcrossScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &ClusterSubAcrossScript;
%let FullAPName=%str(Cluster Subjects Across Study Sites);

%let FilterVars =;
%if %length(&ageflag) %then %do;
	%let FilterVars = &FilterVars.:&trAgeDifference,;
%end;
%if &heightflag %then %do;
	%let FilterVars = &FilterVars.:&trHeightDifference,;
%end;
%if &weightflag %then %do;
	%let FilterVars = &FilterVars.:&trWeightDifference,;
%end;

* put JSL statements;
data _null_;
   file "&ClusterSubAcrossScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put "//!";
	put "Names Default to Here(1);";
	put "Here:DataList = {};";
	put "Here:TabButtonList = {};";
	put "Here:TabCodeList = {};";
	put "Here:TabAfterCodeList = {};";
	put "Here:TabOpenList = {};";
	put "Here:CreateReportList = {};";
	put "Here: TabsSectionFlag = 1;";

  	put "Here:Select=Function({},{default local},";
	put "ctable = Current Data Table();";
	put "If(Is Empty(ctable),";
	put "   New Window(locale:get(""JMPClinicalMessage_Text""), showmenu(0),showtoolbars(0),";
	put "      tb = TextBox(locale:get(""ErrorToRunOpenInFocus_Text"")),";
	put "   	  tb << Font Color(3)";
	put "   );";
	put "   Throw()";
	put ");";
	put "subname = ""&SUBJECTLabel"";";
	put "subname2 = ""&SUBJECTLabel 2"";";
	put "foundit = 1;";
	put "foundit2 = 1;";
	put "try(scol = Column(subname),foundit = 0);";
	put "try(scol2 = Column(subname2),foundit2 = 0);";
	put "selsub = {};";
	put "nsel = 0;";
	put "	for each row(";
	put "		if(selected(rowstate())==1,";
	put "			if(!Contains(selsub,scol()[]),";
	put "				nsel++;";
	put "				InsertInto(selsub,scol()[])";
	put "			);";
	put "			if(foundit2 == 1,";
	put "				if(!Contains(selsub,scol2()[]),";
	put "					nsel++;";
	put "					InsertInto(selsub,scol2()[])";
	put "				);";
	put "			);";
	put "	);";	   
	put ");";
	put "Try(Here:alldat = open(""&ClientOutPath.&OutDataName_sum..sas7bdat"", invisible);";
	put "   Here:alldat << Select Where( Contains(selsub, Column(subname)[]));";
	put "   Current Data Table(Here:alldat);";
	put " );";
	put ");";

	%if %sysfunc(exist(mindist)) %then %do;	
		put "Here:fbox0=Function({},{default local},";
		put "Here:mindist = open(""&ClientOutPath.&OutDataName_min..sas7bdat"",invisible);";
		put "Here:alldist = open(""&ClientOutPath.&OutDataName_all..sas7bdat"",invisible);";
		put "Column(Here:mindist, ""&COVIDlabel"") << Label(1);";
		put "Column(Here:alldist, ""&SubjectLabel"") << Label(1);";
		put "Column(Here:alldist, ""&SubjectLabel2"") << Label(1);";
		put "Here:alldist << Color or Mark by Column(:&trSiteFlag, Color Theme(""JMP Default""));";
		put "Here:allDistgraphs=VlistBox(";
		%if %length(&yeardiff) %then %do;
			put "TextBox(locale:fill(locale:get(""EnrollmentOccOverXAPersThatEnrMultTimesMsg_Text""), {""&yeardiff &yeardifffl""}),<<Set Wrap(900),<< Font Color(5)),";
			put "TextBox(""""),";
		%end;
		put "HListBox(Data Filter Context Box(";
		put "HListBox(Here:alldist << Data Filter(<<Conditional(1),local, Mode( Select( 0 ), Show( 1 ), Include( 1 ) ),Add Filter(columns( &FilterVars :&trSiteFlag ))),";
		put "		Platform(Here:alldist, Here:G1 = Graph Builder(";
		put "			Automatic Recalc(1),";
		put "			Auto Stretching( 0 ),";	
		put "			Show Control Panel( 0 ),";
		put "			Show Legend( 0 ),";
		put "			Variables( X( :&COVIDlabel ), Y( :&trDistance )),";
		put "			Elements(";
		put "				Box Plot(";
		put "					X,";
		put "					Y,";
		put "					Legend( 2 ),";
		put "					Jitter( 1 ),";
		put "					Outliers( 1 ),";
		put "					Box Style( ""Solid"" )";
		put "				),";
		put "			),";
		put "			SendToReport(";
		put "				Dispatch(";
		put "					{},";
		put "					""Graph Builder"",";
		put "					OutlineBox,";
		put "					{Set Title( locale:get(""BetweenSubjectDistancesbyCovariateSubgroup_Text"") )}";
		put "				),";
		put "				Dispatch(";
		put "					{},";
		put "					""&COVIDlabel"",";
		put "					ScaleBox,";
		put "					{Show Major Ticks( 0 ), Rotated Labels( ""Angled"" )}";
		put "				),";
		put "				Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
		put "				Dispatch(";
		put "					{},";
		put "					""Graph Builder"",";
		put "					FrameBox,";
		put "					{Marker Size( 2 ), Transparency( 0.75 )}";
		put "				)";
		put "			)";
		put "		)))),";
		put "		Here:G2 = Here:mindist << Graph Builder(";
		put "			Size( 353, 497 ),";
		put "			Automatic Recalc(1),";
		put "			Auto Stretching( 0 ),";	
		put "			Show Control Panel( 0 ),";
		put "			Show Legend( 0 ),";
		put "			Variables( Y( :&trDistance )),";
		put "			Elements(";
		put "				Box Plot(";
		put "					Y,";
		put "					Legend( 2 ),";
		put "					Jitter( 1 ),";
		put "					Outliers( 1 ),";
		put "					Box Style( ""Outlier"" )";
		put "				),";
		put "				Points( Y,Legend( 3 ), Jitter( 1 ) )";
		put "			),";
		put "			SendToReport(";
		put "				Dispatch(";
		put "					{},";
		put "					""Graph Builder"",";
		put "					OutlineBox,";
		put "					{Set Title( locale:get(""MinimumBetweenSubjectDistancesforeachCovSubgrp_Text"") )}";
		put "				),";
		put "				Dispatch( {}, """", ScaleBox, {Rotated Labels( ""Automatic"" )} ),";
		put "				Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
		put "				Dispatch(";
		put "					{},";
		put "					""Graph Builder"",";
		put "					FrameBox,";
		put "					{Marker Size( 2 ), Transparency( 0.75 )}";
		put "				)";
		put "			)";
		put "		),";		
		put ");";
		put "));";
		put "InsertInto(DataList,{{""Here:mindist"",""Here:alldist""}});";
		put "InsertInto(TabButtonList, expr(locale:get(""BetweenSubjectDistanceSummary_Text"")));";
		put "InsertInto(TabCodeList,expr(Here:fbox0()));";
		put "InsertInto(TabAfterCodeList, "" "");";
		put "InsertInto(TabOpenList,1);";
		put "InsertInto(Here:CreateReportList,{{Here:G1, Here:G2}});";
	%end;
	%let i = 1;
	%let tFlag = 0;
	%do %while(%length(%scan(%nrbquote(&SiteList),&i,'~')));
		%let tempsite = %scan(%nrbquote(&SiteList),&i,'~');	
		%let tempsiten = &i;	
		%if ^%index(%bquote(&SitesNoVariability), %bquote(&tempsite)) %then %do;
			put "Here:fbox&tempsiten=Function({},{default local},";
			put "Here:ddist&tempsiten = open(""&ClientOutPath.&&&OutDataName_&tempsiten...sas7bdat"",invisible);";
			put "Here:dpair&tempsiten = open(""&ClientOutPath.&&&OutDataNameP_&tempsiten...sas7bdat"",invisible);";	
			put "Here:dpair&tempsiten << Color or Mark by Column(:&trSiteFlag, Color Theme(""JMP Default""));";
			put "Column(Here:dpair&tempsiten, ""&SubjectLabel"") << Label(1);";
			put "Column(Here:dpair&tempsiten, ""&SubjectLabel2"") << Label(1);";
			put "Here:Dm&tempsiten=VListBox(";	   
			%if %quote(&tempsite) = %quote(&MinDistSite) %then %do;			
				put "textbox(locale:get(""ThisCovSubgrpHasTheMinBetwSubjDist_Text""),<<Set Wrap(900),<< Font Color(5)),"; 
				put "textbox("""",<<Set Wrap(900),<< Font Color(5)),"; 
			%end; 
			put "HListBox(Data Filter Context Box(";
			put "HListBox(Here:dpair&tempsiten << Data Filter(<<Conditional(1),local, Mode( Select( 0 ), Show( 1 ), Include( 1 ) ),Add Filter(columns( &FilterVars :&trSiteFlag ))),";
			put "		Platform(Here:dpair&tempsiten, Here:G1_&i = Graph Builder(";
			put "			Size( 353, 497 ),";
			put "			Automatic Recalc(1),";
			put "			Auto Stretching( 0 ),";      
			put "			Show Control Panel( 0 ),";
			put "			Show Legend( 0 ),";
			put "			Variables( Y( :&trDistance )),";
			put "			Elements(";
			put "				Box Plot(";
			put "					Y,";
			put "					Legend( 2 ),";
			put "					Jitter( 1 ),";
			put "					Outliers( 1 ),";
			put "					Box Style( ""Outlier"" )";
			put "				),";
			put "				Points( Y, Legend( 3 ), Jitter( 1 ) )";
			put "			),";
			put "			SendToReport(";
			put "				Dispatch(";
			put "					{},";
			put "					""Graph Builder"",";
			put "					OutlineBox,";
			put "					{Set Title( locale:get(""BetweenSubjectDistances_Text"") )}";
			put "				),";
			put "				Dispatch( {}, """", ScaleBox, {Rotated Labels( ""Automatic"" )} ),";
			put "				Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
			put "				Dispatch(";
			put "					{},";
			put "					""Graph Builder"",";
			put "					FrameBox,";
			put "					{Marker Size( 2 ), Transparency( 0.75 )}";
			put "				)";
			put "			)";
			put "		)))),";
			put "		Here:G2_&i = Here:ddist&tempsiten << Hierarchical Cluster(";
			put "       Hide Distance Messages(1),";
			put "			Automatic Recalc(1),";
			put "			Y(&&SiteClusterVars&tempsiten),";
			put "			Label( :&SUBJECTLabel ),";
			put "			Method( ""&HierClustMethod"" ),";
			put "			Standardize( 0 ),";
			put "			Distance Matrix( 1 ),";
			put "			Color Clusters( 1 ),";
			put "			Distance Graph( 0 ),";
			put "			Legend( 1 ),";
			put "			Dendrogram Scale( ""Distance Scale"" ),";
			put "			Two Way Clustering,";
			put "			Color Map( {"""", {{255, 0, 0}, {255, 255, 255}}} ),";
			put "			SendToReport(";
			put "			Dispatch(";
			put "				{""Dendrogram""},";
			put "				""Clust Dendro"",";
			put "				FrameBox,";
			put "				{Frame Size( 45, 352 )}";
			put "			),";
			put "			Dispatch( {""Dendrogram""}, """", CellPlotBox( 2 ), {Frame Size( 349, 352 ), Color Theme({"""", {{255, 0, 0}, {255, 255, 255}}})} ),";
			put "			Dispatch(";
			put "				{""Dendrogram""},";
			put "				""Clust Dendro"",";
			put "				FrameBox( 2 ),";
			put "				{Frame Size( 200, 352 )}";
			put "			),";
			put "			Dispatch(";
			put "				{""Dendrogram""},";
			put "				""Clust Dendro"",";
			put "				FrameBox( 3 ),";
			put "				{Frame Size( 349, 155 )}";
			put "			),";
			put "			Dispatch(";
			put "				{""Dendrogram""},";
			put "				""Clust Dendro"",";
			put "				FrameBox( 4 ),";
			put "				{Frame Size( 349, 59 )}";
			put "			))";				
			put "		);";
			put ")));";
			put "InsertInto(DataList,{{""Here:ddist&tempsiten"", ""Here:dpair&tempsiten""}});";
			put "InsertInto(TabButtonList, expr(""&tempsite""));";
			put "InsertInto(TabCodeList,expr(Here:fbox&tempsiten()));";
			put "InsertInto(TabAfterCodeList, "" "");";
			put "InsertInto(Here:CreateReportList,{{Here:G1_&i,Here:G2_&i}});";
			
			%if %quote(&tempsite) = %quote(&MinDistSite) %then %do;
				put "InsertInto(TabOpenList,1);";
			%end;
			%else %do;
				put "InsertInto(TabOpenList,0);";
				%let tFlag = 1;
			%end;
		%end;
		%else %do;
			put "Here:fbox&tempsiten=Function({},{default local},";
			put "VListBox(TextBox(locale:get(""AllVarsAvForAnHadNoVarWiThisCovGrpThisImpMsg_Text""),<<Set Wrap(900),<< Font Color(5)));"; 
		   put ");";
			put "InsertInto(DataList,{{""""}});";
			put "InsertInto(TabButtonList, expr(""&tempsite""));";
			put "InsertInto(TabCodeList,expr(Here:fbox&tempsiten()));";
			put "InsertInto(TabAfterCodeList, "" "");";
			put "InsertInto(TabOpenList,1);";
			put "InsertInto(Here:CreateReportList,{{Here:G1_&i,Here:G2_&i}});";
		%end;
		%let i = %eval(&i+1);
   %end;
	%if &tFlag=1 %then %do;
		put "Here:TabsSectionFlag = 1;";
	%end;
  
	put "Here:ppdd =  Function({},{},";
	put "Here:Select();";
	put "Include(""$CLINICAL_HOME\ProcessLibrary\PatientProfiler.jsl"",<< New Context,<< ""Names Default To Here""); Close(Here:alldat, nosave););";
	put "Here:spdd =  Function({}, {},";
	put "Here:Select();";
	put "Include(""$CLINICAL_HOME\ProcessLibrary\ShowSubjects.jsl""); Close(Here:alldat, nosave););";
	put "Include(""$CLINICAL_HOME\JSLFiles\Fraud Functions.jsl"");";
	put "Here:ActionButtonList= {{expr(locale:get(""ProfileSubjects_Text"")),expr(locale:get(""ShowSubjects_Text"")),";
	put "   expr(locale:get(""ShowRowsinHeatMap_Text"")),expr(locale:get(""SubsetClusteringRows_Text"")),expr(locale:get(""RevertClustering_Text""))}};";
	put "Here:ActionCodeList= {{Here:ppdd(),Here:spdd(),Here:ShowRow(), Here:SubsetClustering(),Here:RevertClustering()}};";
	put "Here:ActionButtonDesc = {expr(locale:get(""SelectRecordsthenClick_Text""))};";

%let includeds = %substr(%quote(&includeds), 1, %eval(%length(&includeds) - 1));
%PackageInsertFile(jsl code from &ProcessName, &OutPath, &ClusterSubAcrossScriptName, "&includeds", yes);
%PackageInsertDataset(%nrbquote(%LocaleFill(%nrbquote(%LocaleGet(DataFromX_Text)),
	%nrbquote(%LocaleGet(&ProcessName._Label)))), outlib,  &OutDataName_sum, no);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend ClusterSubjectsAcrossSite;
%ClusterSubjectsAcrossSite;

************************************;
************************************;
************************************;