/*
JMP Clinical Analytical Process

Name: Multivariate Inlier
Authors: Richard C. Zink, SAS Institute Inc.

Description: 

This process calculates Mahalanobis Distance to identify multivariate inliers and outliers
	
*/

/*
-----------------------------------------------------------
Copyright (C) 2012 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;

%global StudyName InPath AInPath SubjectFilter TrtVar TrtUse MacroPath OutName MultivariateInlierScript MultivariateInlier_OutData OutPath 
        ServerOutPath PopulationSet MissMax FindDistFl InterDistFl EventDistFl DelMissFl WhereStmt WhereForReport AgeSexFl;

*ProcessBody;

%macro MultivariateInlier;

*initialization and argument checking;

%let ProcessName = MultivariateInlier; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%let WhereForReport=%nrbquote(&WhereStmt);
%if &exiterror %then %goto exit;

%if &AgeSexFl = No and &EventDistFl = No and &InterDistFl = No and &FindDistFl = No %then %do;
	%put ERROR: At least one domain type must be selected for analysis.;
	%let exiterror = 1;
	%goto exit;
%end;

%if %sysfunc(findw(%upcase(&ADSLVARS), AGE, ' ')) %then %let ageflag = age; 
%else %let ageflag = ;

%if %sysfunc(findw(%upcase(&ADSLVARS), SEX, ' ')) %then %let sexflag = sex;
%else %let sexflag = ;

%let _tid_ = %sysfunc(open(&in_adsl));
%let vn = %sysfunc(varnum(&_tid_,USUBJID));
%let USubLen = %sysfunc(varlength(&_tid_,&vn));
%let _tid_ = %sysfunc(close(&_tid_));

%if ^%length(&findingsdomains) %then %do;
	%put ERROR: No Findings domains are available for the current study.;
	%let exiterror = 1;
	%goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), ARM, ' ')) %then %do;
	%put ERROR: ARM is not available in either DM, ADDM or ADSL.;
	%let exiterror = 1;
	%goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), SITEID, ' ')) %then %do;
	%put ERROR: SITEID is not available in either DM, ADDM or ADSL.;
	%let exiterror = 1;
	%goto exit;
%end;

%if %index(%upcase(&in_adsl), ADSL) and %sysfunc(findw(%upcase(&ADSLVARS), COUNTRY, ' ')) = 0 and %sysfunc(findw(%upcase(&DMVARS), COUNTRY, ' ')) > 0 %then %do;
	proc sort data = &in_adsl out = subject(keep = usubjid siteid arm &ageflag &sexflag);
		by usubjid;
	run;

	proc sort data = &in_dm out = dm;
		by usubjid;
	run;

	data subject;
		merge subject(in = ina) dm(keep = usubjid country);
		by usubjid;
		if ina;
	run;
%end;
%else %if %sysfunc(findw(%upcase(&ADSLVARS), COUNTRY, ' ')) %then %do;
	proc sort data = &in_adsl out = subject(keep = usubjid siteid arm country &ageflag &sexflag);
		by usubjid;
	run;
%end;
%else %do;
	%let trNotProv = %nrbquote(%LocaleGet(NotProvided_Text));
	%let trcountry = %nrbquote(%LocaleGet(Country_Text));
	proc sort data = &in_adsl out = subject(keep = usubjid siteid arm &ageflag &sexflag);
		by usubjid;
	run;
	
	data subject;
	   length Country $ 24;
	   set subject;
	   country = symget('trNotProv');
	   label country = "&trcountry";
	run;
%end;

proc sql noprint;
   create table subpairs as
      select aa.usubjid, bb.usubjid2, aa.siteid
      from subject as aa, subject(rename = (usubjid = USUBJID2 siteid = siteid2)) as bb 	
      where aa.usubjid < bb.usubjid2 and siteid = siteid2	
      order by aa.usubjid, bb.usubjid2;
quit;

%let availabledomains = ;

%let trUnscheduled = %nrbquote(%LocaleGet(Unscheduled_Text));

*** create by lists for each domain ***;
%if &FindDistFl = Yes %then %do;
	%let i = 1;
	%do %while(%length(%scan(&findingsdomains,&i,' ')));
		%let temp = %scan(&findingsdomains,&i,' ');

		%if %sysfunc(indexw(&&&temp.VARS, VISITNUM, ' ')) and %sysfunc(indexw(&&&temp.VARS, &temp.STRESN, ' ')) %then %do;
			%let availabledomains = &availabledomains &temp;
			%if %sysfunc(indexw(&&&temp.VARS, &temp.TPTNUM, ' ')) %then %let tpt = &temp.tptnum;
			%else %let tpt =;

			proc sort data = &&in_&temp(where = (not missing (&temp.STRESN))) out = &temp(keep = usubjid &temp.testcd &temp.STRESN VISITNUM &tpt);
				by usubjid &temp.testcd visitnum &tpt;
			run;

			*** Delete unscheduled records ***;
			%if %sysfunc(indexw(&&&temp.VARS, VISIT, ' ')) %then %do;
				data &temp;
					set &temp;
					if index(upcase(visit), "UNSCHEDULED") or kindex(kupcase(visit), "%kupcase(&trUnscheduled)") then delete;
				run;
			%end;

			ods exclude all;
			ods noresults;
			proc means noprint nway data = &temp;
				class usubjid &temp.testcd visitnum &tpt;
				var &temp.STRESN;
				output out = &temp(drop = _type_ _freq_) mean = &temp.STRESN;
			run;
			ods select all;
			ods results;

			data &temp;
				set &temp;
				%if %length(&tpt) %then %do;
					level = cats(&temp.testcd,"_","V",visitnum,"_","T",&tpt);         
				%end;
				%else %do;
					level = cats(&temp.testcd,"_","V",visitnum);
				%end;
			run;

			proc sort data = &temp out = &temp;
				by usubjid level;
			run;

			proc transpose data = &temp out = &temp(drop = _name_ _label_);
				by usubjid;
				var &temp.STRESN;
				id level;
				idlabel level;
			run;              
		%end;
		%let i = %eval(&i+1);
	%end;
%end;

%macro INTEVENT(list = );
	%let i = 1;
	%do %while(%length(%scan(&list,&i,' ')));
		%let temp = %scan(&list,&i,' ');

		%if %sysfunc(indexw(&&&temp.VARS, &temp.DECOD, ' ')) %then %do;
			%let availabledomains = &availabledomains &temp;

			proc sort data = &&in_&temp(where = (not missing (&temp.DECOD))) out = &temp;
				by usubjid &temp.DECOD;
			run;
         
			*** Delete prespecified not occuring records ***;
			data &temp(keep = usubjid &temp.DECOD count);
				set &temp;
				count = 1;
				%PRESPECIFY(dom = &temp);
			run;

         proc freq noprint data = &temp;
            tables &temp.DECOD / out = outname(drop = count percent);
         run;
                  
         data outname;
            length &temp.short $ 8;
            set outname;
            &temp.short = cats("&temp",_n_);   
         run;

         proc sort data = &temp;
            by usubjid &temp.DECOD;
			run;

			ods exclude all;
			ods noresults;
			proc means noprint nway data = &temp;
				class usubjid &temp.DECOD;
				var count;
				output out = &temp(drop = _type_ _freq_) n = count;
			run;
			ods select all;
			ods results;

			proc sql noprint;      
				create table events as
					select distinct &temp.DECOD
					from &temp
					order by &temp.DECOD;		
			quit;

			proc sql;
				create table events1 as
				select aa.usubjid, bb.&temp.DECOD
				from subject as aa, events as bb
				order by aa.usubjid, bb.&temp.DECOD;
			quit;

			data &temp;
			  merge &temp(in = ina) events1(in = inb);
			  by usubjid &temp.DECOD;
			  if inb and not ina then count = 0;		  
			run;

         proc sort data = &temp;
            by &temp.decod;
         run;
           
         data &temp;
            merge &temp outname;
            by &temp.decod;
         run;

			proc datasets nolist;
				delete events events1 outname;
			quit;
			
         proc sort data = &temp;
            by usubjid &temp.decod;
         run;			
           
			proc transpose data = &temp out = &temp(drop = _name_ _label_);
				by usubjid;
				var count;
				id &temp.short; 
				idlabel &temp.DECOD;
			run;   
		%end;

		%let i = %eval(&i+1);
	%end;
%mend INTEVENT;

%if &EventDistFl = Yes %then %do;
	%INTEVENT(list = &eventsdomains);
%end;

%if &InterDistFl = Yes %then %do;
	%INTEVENT(list = &interventionsdomains); 
%end;

%let trMaleGender = %nrbquote(%LocaleGet(MaleGender_Text));
%let trM = %nrbquote(%LocaleGet(M_Text));
%let trF = %nrbquote(%LocaleGet(F_Text));
data all;
   merge subject(in = ina) &availabledomains;
   by usubjid;
   if ina;
   %if %length(&sexflag) %then %do;
      if sex = "M" or sex="&trM" then MALE = 1;
      else if sex = "F" or sex ="&trF" then MALE = 0;
      drop sex;
      label male = "&trMaleGender";
   %end;
	%if &AgeSexFl = No %then %do;
		%if %length(&ageflag) %then %do;
      	drop age;
		%end;
		%if %length(&sexflag) %then %do;
			drop male;
		%end;
	%end;    
run;
%NObsVars(all);
%let TotalSubs = &nobs;

proc means nmiss noprint data = all;
	output out = nmiss(drop = _type_ _freq_) nmiss = / autoname;
run;

proc transpose data = nmiss out = nmiss;
run;

%let trVariable = %nrbquote(%LocaleGet(Variable_Text));
%let trLabel = %nrbquote(%LocaleGet(Label_Text));
%let trNOSMV = %nrbquote(%LocaleGet(NumberofSubjectsMissingValues_Text));
%let trPOSMV = %nrbquote(%LocaleGet(PercentageofSubjectsMissingValues_Text));
%let trMissFlag = %nrbquote(%LocaleGet(MissingFlag_Text));

data nmiss;
   set nmiss(rename = (col1 = nummiss)) end = eof;
   _name_ = substr(_name_, 1, length(_name_) - 6);
   pctmiss = round(nummiss / &TotalSubs * 100, 0.01);
   %if %upcase(&DelMissFl) = YES %then %do; 
   	if pctmiss > 0 then missflag = 2;
   %end;
   %else %do;
   	if pctmiss >= &MissMax then missflag = 2;
   %end;
   else if pctmiss ne 0 then missflag = 1;
   else if pctmiss = 0 then missflag = 0;
   if eof then call symput("TotalVars", trim(left(_n_)));
   label _name_ = "&trVariable" _label_ = "&trLabel" nummiss = "&trNOSMV" pctmiss = "&trPOSMV" missflag = "&trMissFlag";
run;

*** degree of freeom is number of variables contributing to maha scores;
%let DegFree = 0;
data _null_;
   set nmiss(where = (missflag in (0,1))) end = eof;
   if eof then call symput("DegFree", trim(left(_n_)));
run;

%let MissyVar = 0;
data _null_;
   set nmiss(where = (missflag in (1))) end = eof;
   if eof then call symput("MissyVar", trim(left(_n_)));
run;

%let RemovedVars = ;
data _null_;
   set nmiss(where = (missflag = 2)) end = eof;
   if eof then call symput("RemovedVars", trim(left(_n_)));
run;

%if ^%length(&RemovedVars) %then %let RemovedVars = 0;
%if ^%eval(&TotalVars - &RemovedVars) %then %do;
	%put ERROR: No variables remain for analysis after deleting variables with at least &MissMax percent missing data.;
	%let exiterror = 1;
	%goto exit;
%end;
%let OtherVars = %eval(&MissyVar);

%let MissMaxVars = ;
%let MissMaxLabels = ;
%let MissLabels = ;
proc sql noprint;
	select _name_ 
		into :MissMaxVars separated by ' '
		from nmiss
		where missflag = 2
		order by _name_;
/*	select _label_ 
		into :MissMaxLabels separated by ' '
		from nmiss
		where missflag = 2
		order by _name_;		
	select _label_ 
		into :MissLabels separated by ' '
		from nmiss
		where missflag = 1
		order by _name_; */
quit;

ods exclude all;
ods noresults;
proc princomp data = all(drop = &MissMaxVars) std noprint out = prin;
   id usubjid;
   var _numeric_;
run;
ods select all;
ods results;   

%let trMahDist = %nrbquote(%LocaleGet(MahalanobisDistance_Text));
%let trNLMahDist = %nrbquote(%LocaleGet(NaturalLogMahalanobisDistance_Text));

data prin(keep = usubjid mahadist logmahadist);
   set prin;
   MahaDist = sqrt(uss(of prin:));
   if MahaDist > 0 then LogMahaDist = log(MahaDist);
   label MahaDist = "&trMahDist" LogMahaDist = "&trNLMahDist";
run; 

data all;
   merge all prin;
   by usubjid;
run;

data _null_;
   set all(where = (MahaDist = .)) end = eof;
   if eof then call symput("SubMissMaha", trim(left(_n_)));
run;

%let Ref = %sysfunc(sqrt(&DegFree));
%let LogRef = %sysfunc(log(&Ref));

/*
ods exclude all;
ods noresults;
proc distance reponly prefix = S_ data = all shape = square out = distance;
   var interval(_numeric_ / std = std);
   id usubjid;
run;
ods select all;
ods results;   

proc datasets nolist;
   delete outdat;
quit;

proc transpose data = distance out = distance;
   var _numeric_;
   by usubjid;
run;

data distance(drop = _name_);
   length USUBJID2 $ &USubLen;
   set distance(where = (col1 > .));
   USUBJID2 = substr(_NAME_,3);
   if usubjid = usubjid2 then delete;   
run;

proc sort data = distance;
   by usubjid usubjid2;
run;

data distance;
   merge subpairs(in = ina) distance;
   by usubjid usubjid2;
   if ina;
run;


data outlib.subdist;
   set distance;
run;

proc print data = distance;
run;
%goto exit;      
*/

%let MahaLabel = MahaDist;
%CheckLabel(all,MahaLabel);
%let LogMahaLabel = LogMahaDist;
%CheckLabel(all,LogMahaLabel);
%let ARMLabel = ARM;
%CheckLabel(all,ARMLabel);
%let SUBJECTLabel = USUBJID;
%CheckLabel(all,SUBJECTLabel);
%let COUNTRYLabel = COUNTRY;
%CheckLabel(all,COUNTRYLabel);
%let SITEIhlabel = SITEID;
%CheckLabel(all,SITEIhlabel);

%let MultivariateInlier_OutData_1 = &MultivariateInlier_OutData;	   
%PrepOutDS(MultivariateInlier_OutData_1,OutDataName_1,mahadist);
%let MultivariateInlier_OutData_2 = &MultivariateInlier_OutData;	   
%PrepOutDS(MultivariateInlier_OutData_2,OutDataName_2,mahdistnmiss);

data &MultivariateInlier_OutData_1;
	set all;
run;

data &MultivariateInlier_OutData_2;
	set nmiss(where = (missflag > 0));
run;
%NObsVars(&MultivariateInlier_OutData_2);
%let MissFlag = &nobs;

%let includeds = &OutDataName_1..sas7bdat,&OutDataName_2..sas7bdat;

proc datasets nolist;
   delete all nmiss;
quit;		

* create JSL file;
%PrepOutFile(MultivariateInlierScript,MultivariateInlierScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &MultivariateInlierScript;
%let FullAPName=%str(Multivariate Inliers and Outliers);

%put MOOO = &OtherVars;

* put JSL statements;
data _null_;
   file "&MultivariateInlierScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put "//!";
	put "Names Default to Here(1);";
	put "Here:DataList = {};";
	put "Here:TabButtonList = {};";
	put "Here:TabCodeList = {};";
	put "Here:TabAfterCodeList = {};";
	put "Here:TabOpenList = {};";
	put "Here:CreateReportList = {};";
	
	put "try(Here:SubjectFilter=""\[&SubjectFilter]\"");";
	put "try(Here:WhereState=""\[&WhereForReport]\"");";
	put "try(Here:PopulationSet=""\[&PopulationSet]\"");";
	put "try(Here:WhereFilters=""Additional Filter to include Subjects: "" || Here:WhereState);";	
		
	put "Here:fbox=Function({},{default local},";
	put "Here:dmaha=open(""&ClientOutPath.&OutDataName_1..sas7bdat"",invisible);";
	put "Column(""&SubjectLabel"") << Label(1);";
		
	put "if(""&MahaLabel"" == ""Mahalanobis Distance"", trMahaLabel = locale:get(""MahalanobisDistance_Text""),";
	put "	trMahaLabel = ""&MahaLabel"");";
	put "if(""&LogMahaLabel"" == ""Natural Log(Mahalanobis Distance)"", trLogMahaLabel = locale:get(""NaturalLogMahalanobisDistance_Text""),";
	put "	trLogMahaLabel = ""&LogMahaLabel"");";
	put "if(""&SITEIhlabel"" == ""Study Site Identifier"", trSITEIhlabel = locale:get(""StudySiteIdentifier_Text""),";
	put "	trSITEIhlabel = ""&SITEIhlabel"");";
	
	put "VListBox(";
	put "Here:B=textbox(locale:fill(locale:get(""MahDistCompFromXofXPossVarXVarRemSlidXPerc_Text""), {""&DegFree"", ""&TotalVars"", ""&RemovedVars"", ""&MissMax""}),<<Set Wrap(1000),<< Font Color(5)),"; 
	%if &OtherVars %then %do;
		put "Here:C=textbox(locale:fill(locale:get(""XVarsMissDatRatesltXPercBecXVarsXSubjNoMahDist_Text""), {""&OtherVars"", ""&MissMax"", ""&OtherVars"", ""&SubMissMaha""}),<<Set Wrap(900),<< Font Color(5)),"; 
	%end;
	%else %do;
	put "Here:C=textbox("" "");";
	%end;
	
	put "textbox(""""),"; 
	put "HListBox(";
	put "		Here:MD=graph Builder(";
	put "       Automatic Recalc(1),";
	put "			Auto Stretching( 0 ),";
	put "			Show Legend( 0 ),";
	put "			Show Control Panel( 0 ),";
	put "			Variables( Y( :Name(""&MahaLabel"") ) ),";
	put "			Elements(";
	put "				Box Plot(";
	put "					X,";
	put "					Y,";
	put "					Legend( 3 ),";
	put "					Jitter( 1 ),";
	put "					Outliers( 1 )";
	put "				),";
	put "          Points( Y, Legend( 4 ), Jitter( 1 ) )";
	put "			),";
	put "			SendToReport(";
	put "          Dispatch({},""Graph Builder"",OutlineBox, {Set Title( locale:fill(locale:get(""XforallSites_Text""), {trMahaLabel}) )}),";
	put "				Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
	put "				Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Size( 3 )} ),";
	put "				Dispatch({},""&MahaLabel"",ScaleBox, {Add Ref Line( &Ref, Dashed, ""Red"" )})";
	put "			)";
	put "		),";			
	put "		Here:MDS=Graph Builder(";
	put "       Automatic Recalc(1),";
	put "			Auto Stretching( 0 ),";	
	put "			Show Legend( 0 ),";
	put "			Show Control Panel( 0 ),";
	put "			Variables( X( :Name(""&SITEIhlabel"") ), Y( :Name(""&MahaLabel"") ) ),";
	put "			Elements(";
	put "				Box Plot(";
	put "					X,";
	put "					Y,";
	put "					Legend( 3 ),";
	put "					Jitter( 1 ),";
	put "					Outliers( 1 )";
	put "				)";
	put "			),";
	put "			SendToReport(";
	put "          Dispatch({},""Graph Builder"",OutlineBox, {Set Title( locale:fill(locale:get(""XvsX_Text""), {trMahaLabel, trSITEIhlabel}) )}),";
	put "				Dispatch( {}, ""&SITEIhlabel"", ScaleBox, {Show Major Ticks( 0 )} ),";
	put "				Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
	put "				Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Size( 3 )} ),";
	put "				Dispatch({},""&MahaLabel"",ScaleBox, {Add Ref Line( &Ref, Dashed, ""Red"" )})";
	put "			)";
	put "		),";			
	put "	Here:Filter=   Current Data Table() << Data Filter(<<Conditional(1),";
	put "	   	Add Filter(";
	put "	        	Columns(:Name(""&COUNTRYLabel""), :Name(""&SITEIhlabel""))"; 
	put "	      ),";
	put "	  		Mode( Show(1), Include(1) )";
	put "	   )";
	put "),"; 
	put "HListBox(";
	put "		Here:MDN=	Graph Builder(";
	put "       Automatic Recalc(1),";
	put "			Auto Stretching( 0 ),";
	put "			Show Legend( 0 ),";
	put "			Show Control Panel( 0 ),";
	put "			Variables( Y( :Name(""&LogMahaLabel"") ) ),";
	put "			Elements(";
	put "				Box Plot(";
	put "					X,";
	put "					Y,";
	put "					Legend( 3 ),";
	put "					Jitter( 1 ),";
	put "					Outliers( 1 )";
	put "				),";
	put "          Points( Y, Legend( 4 ), Jitter( 1 ) )";
	put "			),";
	put "			SendToReport(";
	put "          Dispatch({},""Graph Builder"",OutlineBox, {Set Title( locale:fill(locale:get(""XforallSites_Text""), {trLogMahaLabel}) )}),";
	put "				Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
	put "				Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Size( 3 )} ),";
	put "				Dispatch({},""&LogMahaLabel"",ScaleBox, {Add Ref Line( &LogRef, Dashed, ""Red"" )})";
	put "			)";
	put "		),";			
	put "		Here:MDNL=	Graph Builder(";
	put "       Automatic Recalc(1),";
	put "			Auto Stretching( 0 ),";	
	put "			Show Legend( 0 ),";
	put "			Show Control Panel( 0 ),";
	put "			Variables( X( :Name(""&SITEIhlabel"") ), Y( :Name(""&LogMahaLabel"") ) ),";
	put "			Elements(";
	put "				Box Plot(";
	put "					X,";
	put "					Y,";
	put "					Legend( 3 ),";
	put "					Jitter( 1 ),";
	put "					Outliers( 1 )";
	put "				)";
	put "			),";
	put "			SendToReport(";
	put "          Dispatch({},""Graph Builder"",OutlineBox, {Set Title( locale:fill(locale:get(""XvsX_Text""), {trLogMahaLabel, trSITEIhlabel}) )}),";
	put "				Dispatch( {}, ""&SITEIhlabel"", ScaleBox, {Show Major Ticks( 0 )} ),";
	put "				Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
	put "				Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Size( 3 )} ),";
	put "				Dispatch({},""&LogMahaLabel"",ScaleBox, {Add Ref Line( &LogRef, Dashed, ""Red"" )})";
	put "			)";
	put "		))";			
	put ")"; *end of Vlist;
	put ");"; 
	put "InsertInto(DataList,{{""Here:dmaha""}});";
	put "TabToDisplay = ""&MahaLabel"";";
	put "if(TabToDisplay == ""Mahalanobis Distance"", TabToDisplay = locale:get(""MahalanobisDistance_Text""););";
	put "InsertInto(TabButtonList, TabToDisplay);";
	put "InsertInto(TabCodeList,expr(Here:fbox()));";
	put "InsertInto(TabAfterCodeList, "" "");";
	put "InsertInto(TabOpenList,1);";
	put "InsertInto(CreateReportList,{{Here:B,Here:C,Here:MD,Here:MDS,Here:MDN,Here:MDNL}});";
	%if %length(&MissFlag) %then %do;
		put "Here:fbox2=Function({},{default local},";
		put "Here:dmahamiss=open(""&ClientOutPath.&OutDataName_2..sas7bdat"",invisible);";
		put "   column(""&trMissFlag"") << Set Modeling Type( Ordinal );";
		put "VListBox(";
		put "textbox(locale:fill(locale:get(""MissFlag1ReflVarsMissDatNotRemDialOptSlidXPerc_Text""), {""&MissMax""}),<<Set Wrap(1000),<< Font Color(5)),"; 
		put "textbox(locale:get(""SubjWithMissValDoNotHaveCompMahDist_Text""),<<Set Wrap(700),<< Font Color(5)),"; 
		put "textbox(locale:get(""MissFlag2ReflVarsMissDatRemPriorCompMahDist_Text""),<<Set Wrap(700),<< Font Color(5)),"; 
		put "textbox(""""),"; 
		put "HListBox(";
		put "Here:Dist=Distribution(";
		put "         Automatic Recalc(1),";
		put "	Weight( :&trNOSMV ),";
		put "	Nominal Distribution( Column( :&trLabel ), Show Counts( 1 ) ),";
		put "	SendToReport(";
		put "		Dispatch(";
		put "			{},";
		put "			""&trLabel"",";
		put "			OutlineBox,";
		put "			{Set Title( locale:get(""VariableswithMissingData_Text"") )}";
		put "		),";
		put "		Dispatch( {}, ""Frequencies"", OutlineBox, {Close( 1 )} )";
		put "	)";
		put "),";
		put "	   Current Data Table() << Data Filter(<<Conditional(1),";
		put "	   	Add Filter(";
		put "	        	Columns(:&trMissFlag)"; 
		put "	      ),";
		put "	  		Mode( Show(1), Include(1) )";
		put "	   )";
		put ")";
		put ");"; 
      put ");"; 
		put "InsertInto(DataList,{{""Here:dmahamiss""}});";
		put "InsertInto(TabButtonList, expr(locale:get(""MissingData_Text"")));";
		put "InsertInto(TabCodeList,expr(Here:fbox2()));";
		put "InsertInto(TabAfterCodeList, "" "");";
		put "InsertInto(TabOpenList,1);";
		put "InsertInto(CreateReportList,{{Here:Dist}});";
	%end;
	%else %do;	
	%end;	
	put "Here:ppdd =  Function({},{},Include(""$CLINICAL_HOME\ProcessLibrary\PatientProfiler.jsl"",<< New Context,<< ""Names Default To Here""));";
	put "Here:cpdd =  Function({},{},Include(""$CLINICAL_HOME\ProcessLibrary\ClusterSubjects.jsl"",<< New Context,<< ""Names Default To Here""));";
	put "Here:fpdd =  Function({},{},Include(""$CLINICAL_HOME\ProcessLibrary\FilterSubjects.jsl"",<< New Context,<< ""Names Default To Here""));";		
	put "Here:spdd =  Function({}, {}, Include(""$CLINICAL_HOME\ProcessLibrary\ShowSubjects.jsl""));";
	put "Here:ActionButtonList= {{expr(locale:get(""ProfileSubjects_Text"")),expr(locale:get(""ShowSubjects_Text"")),expr(locale:get(""ClusterSubjects_Text"")),expr(locale:get(""CreateSubjectFilter_Text""))}};";
	put "Here:ActionCodeList= {{Here:ppdd(),Here:spdd(),Here:cpdd(),Here:fpdd}};";
	put "Here:ActionButtonDesc = {expr(locale:get(""SelectRecordsthenClick_Text""))};";

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &MultivariateInlierScriptName, "&includeds", yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend MultivariateInlier;
%MultivariateInlier;

************************************;
************************************;
************************************;
